@@ -20,17 +20,10 @@
 #include <stringzilla/stringzilla.h>
 
 #if SZ_AVOID_LIBC
-// If we don't have the LibC, the `malloc` definition in `stringzilla.h` will be illformed.
 #ifdef _MSC_VER
 typedef sz_size_t size_t; // Reuse the type definition we've inferred from `stringzilla.h`
-extern __declspec(dllimport) int rand(void);
-extern __declspec(dllimport) void free(void *start);
-extern __declspec(dllimport) void *malloc(size_t length);
 #else
 typedef __SIZE_TYPE__ size_t; // For GCC/Clang
-extern int rand(void);
-extern void free(void *start);
-extern void *malloc(size_t length);
 #endif
 #endif
 
@@ -128,6 +121,14 @@ static void sz_dispatch_table_update_implementation_(sz_capability_t caps) {
     }
 #endif
 
+#if SZ_USE_GOLDMONT
+    if (caps & sz_cap_goldmont_k) {
+        impl->sha256_state_init = sz_sha256_state_init_goldmont;
+        impl->sha256_state_update = sz_sha256_state_update_goldmont;
+        impl->sha256_state_digest = sz_sha256_state_digest_goldmont;
+    }
+#endif
+
 #if SZ_USE_HASWELL
     if (caps & sz_cap_haswell_k) {
         impl->equal = sz_equal_haswell;
@@ -189,6 +190,10 @@ static void sz_dispatch_table_update_implementation_(sz_capability_t caps) {
         impl->hash_state_digest = sz_hash_state_digest_ice;
         impl->fill_random = sz_fill_random_ice;
 
+        impl->sha256_state_init = sz_sha256_state_init_ice;
+        impl->sha256_state_update = sz_sha256_state_update_ice;
+        impl->sha256_state_digest = sz_sha256_state_digest_ice;
+
         impl->sequence_intersect = sz_sequence_intersect_ice;
     }
 #endif
